import requests
import json

from django.db import models
from django.urls import reverse
from django.contrib.auth.models import User
from django.utils import timezone
from datetime import timedelta

from allianceauth.eveonline.models import EveCharacter

from django.utils.translation import ugettext_lazy as _
# Create your models here.

class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app"),
            ("view_public", "Can see public events"),
            ("view_member", "Can see member events"),
            ("create_event", "Can create and edit events"),
            ("manage_event", "Can delete and manage signups"),
        )     

class WebHook(models.Model):
    """Discord Webhook for pings"""
    name = models.CharField(max_length=150)
    webhook_url = models.CharField(max_length=500)
    enabled = models.BooleanField()

    def send_embed(self, embed):
        custom_headers = {'Content-Type': 'application/json'}
        data = '{"embeds": [%s]}' % json.dumps(embed)
        r = requests.post(self.webhook_url, headers=custom_headers,
                            data=data)
        r.raise_for_status()

    class Meta:
        verbose_name = 'Webhook'
        verbose_name_plural = 'Webhooks'

    def __str__(self):
        return '{}'.format(self.name)

class EventSignal(models.Model):
    """Fleet Timer Create/Delete pings"""

    webhook = models.ForeignKey(WebHook, on_delete=models.CASCADE)

    ignore_past_fleets = models.BooleanField(default=True)

    def __str__(self):
        return 'Send Fleets to "{}"'.format(self.webhook.name)

    class Meta:
        verbose_name = 'Fleet Signal'
        verbose_name_plural = 'Fleet Signals'    

class EventHost(models.Model):
    """Fleet Timer Create/Delete pings"""

    community = models.CharField(max_length=150, null=False)
    ingame_channel = models.CharField(max_length=150, blank=True)
    ingame_mailing_list = models.CharField(max_length=150, blank=True)
    fleet_comms = models.CharField(max_length=150, blank=True)
    fleet_doctrines = models.CharField(max_length=150, blank=True)
    website = models.CharField(max_length=150, blank=True)
    discord = models.CharField(max_length=150, blank=True)
    twitch = models.CharField(max_length=150, blank=True)
    twitter = models.CharField(max_length=150, blank=True)
    youtube = models.CharField(max_length=150, blank=True)
    facebook = models.CharField(max_length=150, blank=True)
    details = models.CharField(max_length=150, blank=True)
   
    def __str__(self):
        return str(self.community)

    class Meta:
        verbose_name = 'Host'
        verbose_name_plural = 'Hosts'           

class EventCategory(models.Model):
    # Colors for calendar
    COLOR_BLUE = "blue"
    COLOR_GREEN = "green"
    COLOR_RED = "red"
    COLOR_ORANGE = "orange"  
    COLOR_GREY = "grey"
    COLOR_YELLOW = "yellow"
    COLOR_PURPLE = "purple"

    COLOR_CHOICES = (
        (COLOR_GREEN, _("Green")),
        (COLOR_RED, _("Red")),
        (COLOR_ORANGE, _("Orange")),
        (COLOR_BLUE, _("Blue")),
        (COLOR_GREY, _("Grey")),
        (COLOR_YELLOW, _("Yellow")),
        (COLOR_PURPLE, _("Purple")),

    )
    name = models.CharField(max_length=150)
    ticker = models.CharField(max_length=10) 
    color = models.CharField(max_length=6, choices=COLOR_CHOICES, default='green')

    class Meta:
        verbose_name = 'Category'
        verbose_name_plural = 'Categories'

    def __str__(self):
        return str(self.name)

class EventImport(models.Model):
    """NPSI IMPORT OPTIONS"""
    SPECTRE_FLEET = "Spectre Fleet"
    EVE_UNIVERSITY = "EVE University"
    
    IMPORT_SOURCES = (
        (SPECTRE_FLEET, _("Spectre Fleet")),
        (EVE_UNIVERSITY, _("EVE University")),
        )

    source = models.CharField(
        max_length=32,
        choices=IMPORT_SOURCES,
        help_text="The API source where you want to pull events from"
    )
    
    host = models.ForeignKey(
        EventHost, on_delete=models.CASCADE,
        default=1,
        help_text="The AA host that will be used for the pulled events"
    )
    operation_type = models.ForeignKey(
        EventCategory,
        on_delete=models.CASCADE,
        help_text="Operation type and ticker that will be assigned for the pulled fleets"
    )
    creator = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        default='1',
        help_text="User that has been used to create the fleet (most often the superuser who manages the plugin)"
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text="Event creator main character"
    )
   
    def __str__(self):
        return str(self.source)

    class Meta:
        verbose_name = 'Event Import'
        verbose_name_plural = 'Event Imports'   

class Event(models.Model):
    # visibility
    VISIBILITY_PUBLIC = "public"
    VISIBILITY_MEMBER = "member"
    VISIBILITY_EXTERNAL = "import"

    VISIBILITY_CHOICES = [
        (VISIBILITY_PUBLIC, _("Public access")),
        (VISIBILITY_MEMBER, _("Members only access")),
    ]


    operation_type = models.ForeignKey(
        EventCategory, 
        on_delete=models.CASCADE
    )
    title = models.CharField(
        max_length=200, 
        unique=False
    )
    host = models.ForeignKey(
        EventHost, 
        on_delete=models.CASCADE, 
        default=1
    )
    doctrine = models.CharField(
        max_length=254, 
        default="", 
        blank=True
    )
    formup_system = models.CharField(
        max_length=254, 
        default="", 
        blank=True
    )
    description = models.TextField()
    start_time = models.DateTimeField()
    end_time = models.DateTimeField()
    fc = models.CharField(
        max_length=254, 
        default=""
    )
    visibility = models.CharField(
        max_length=7,
        choices=VISIBILITY_CHOICES,
        default=VISIBILITY_PUBLIC,
        db_index=True,
    )
    created_date = models.DateTimeField(
        default=timezone.now
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL
    )
    user = models.ForeignKey(
        User, 
        on_delete=models.CASCADE
    )

    class Meta:
        unique_together = ['title', 'start_time']

    def duration(self):
        return self.end_time - self.start_time

    def __str__(self):
        return self.title
    
    def get_absolute_url(self):
        return reverse('opcalendar:event-detail', args=(self.id,))

    @property
    def get_html_url(self):
        url = reverse('opcalendar:event-detail', args=(self.id,))
        return f'{url}'

    @property
    def get_html_title(self):
        return f'{self.start_time.strftime("%H:%M")} - {self.end_time.strftime("%H:%M")} <i>{self.host.community}</i> <br> <b>{self.operation_type.ticker} {self.title}</b>'
    @property
    def get_html_operation_color(self):
        return f'{self.operation_type.color}'

    @property
    def get_html_visibility_color(self):
        if self.visibility=="public":
            return "purple"
        if self.visibility=="import":
            return "grey"
        if self.visibility=="member":   
            return "blue"

    def user_can_edit(self, user: user) -> bool:
        """Checks if the given user can edit this timer. Returns True or False"""
        return user.has_perm("opcalendar.manage_event") or (
            self.user == user and user.has_perm("opcalendar.create_event")
        )

class EventMember(models.Model):
    event = models.ForeignKey(Event, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    class Meta:
        unique_together = ['event', 'user']

    def __str__(self):
        return str(self.user)

    



# Generated by Django 3.1.2 on 2020-11-04 11:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0002_auto_20201104_1054'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventHost',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('community', models.CharField(max_length=150)),
            ],
            options={
                'verbose_name': 'Host',
                'verbose_name_plural': 'Hosts',
            },
        ),
    ]



# Generated by Django 3.1.2 on 2020-11-04 11:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0004_event_host'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='host',
            field=models.ForeignKey(default=1, null=True, on_delete=django.db.models.deletion.CASCADE, to='opcalendar.eventhost'),
        ),
    ]



from django.urls import path

from . import views

app_name = 'opcalendar'
urlpatterns = [
    path('index', views.index, name='index'),
    path('', views.CalendarView.as_view(), name='calendar'),
    path('event/new/', views.create_event, name='event_new'),
    path('event/edit/<int:pk>/', views.EventEdit.as_view(), name='event_edit'),
    path('event/<int:event_id>/details/', views.event_details, name='event-detail'),
    path('add_eventmember/<int:event_id>', views.add_eventmember, name='add_eventmember'),
    path('event/<int:event_id>/remove', views.EventDeleteView, name="remove_event"),
]


# Generated by Django 3.1.2 on 2021-01-12 09:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('opcalendar', '0011_eventimport'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventimport',
            name='source',
            field=models.CharField(choices=[('Spectre Fleet', 'Spectre Fleet'), ('EVE University', 'EVE University')], max_length=32),
        ),
    ]



from celery import shared_task
from datetime import datetime
from django.utils import timezone
import pytz
import feedparser
import logging
import re
from dateutil.parser import parse
from .models import Event, EventImport

from ics import Calendar
import requests

logger = logging.getLogger(__name__)

# Import eve uni classes
@shared_task
def import_fleets():
	
	#Clear out all fleets to remove deleted fleets
	Event.objects.filter(visibility="import").delete()

	local_tz = pytz.timezone("UTC") 

	#Get all import feeds
	feeds = EventImport.objects.all()
	
	for feed in feeds:
		if feed.source=="Spectre Fleet":

			#Get fleets from SF RSS
			d = feedparser.parse('https://www.spectre-fleet.space/engagement/events/rss')
			for entry in d.entries:
				##Look for SF fleets only
				if entry.author_detail.name=='Spectre Fleet':
					#Only active fleets
					if not "[RESERVED]" in entry.title:

						date_object = datetime.strptime(entry.published,'%a, %d %b %Y %H:%M:%S %z')
						date_object.strftime('%Y-%m-%dT%H:%M')

						event = Event(
							operation_type=feed.operation_type,
							title=entry.title,
							host=feed.host,
							doctrine="",
							formup_system="",
							description=entry.description,
							start_time=date_object, 
							end_time=date_object, 
							fc="",
							visibility="import",
							user_id = feed.creator.id,
							eve_character_id = feed.eve_character.id
						)
						event.save()

		if feed.source=="EVE University":
			#Get fleets from EVE UNI Ical
			url = "https://portal.eveuniversity.org/api/getcalendar"
			c = Calendar(requests.get(url).text)
			for entry in c.events:
				#Filter only class events as they are the only public events in eveuni
				
				if "class" in entry.name.lower():		
					event = Event(
						operation_type=feed.operation_type,
						title=re.sub("[\(\[].*?[\)\]]", "", entry.name),
						host=feed.host,
						doctrine="",
						formup_system="",
						description=entry.description,
						start_time=datetime.utcfromtimestamp(entry.begin.timestamp).replace(tzinfo=pytz.utc), 
						end_time=datetime.utcfromtimestamp(entry.end.timestamp).replace(tzinfo=pytz.utc), 
						fc="",
						visibility="import",
						user_id = feed.creator.id,
						eve_character_id = feed.eve_character.id
					)
					event.save()



@shared_task
def add(x, y):
    return x + y

   


# calendarapp/utils.py

from datetime import datetime, timedelta, date, timezone
from calendar import HTMLCalendar
from .models import Event
from eventcalendar.helper import get_current_user

from django.utils.translation import ugettext_lazy as _

class Calendar(HTMLCalendar):
	def __init__(self, year=None, month=None, user=None):
		self.year = year
		self.month = month
		self.user = user
		super(Calendar, self).__init__()
		
	# formats a day as a td
	# filter events by day
	def formatday(self, day, events):
		events_per_day = events.filter(start_time__day=day).order_by('start_time')
		d = ''
		
		# Only events for current month
		if day != 0:
			# Parse events
			for event in events_per_day:
				#Display public events
				if event.visibility == "public" or event.visibility == "import" and self.user.has_perm('opcalendar.view_public'):
					#Get past events
					if datetime.now(timezone.utc) > event.start_time:
						d += f'<a class="nostyling" href="{event.get_html_url}"><div class="event {event.get_html_operation_color} past-event {event.visibility}-event">{event.get_html_title}</div></a>'
					if datetime.now(timezone.utc) <= event.start_time:
						d += f'<a class="nostyling" href="{event.get_html_url}"><div class="event {event.get_html_operation_color} {event.visibility}-event">{event.get_html_title}</div></a>'
				if event.visibility == "member" and self.user.has_perm('opcalendar.view_member'):
					#Get past events
					if datetime.now(timezone.utc) > event.start_time:
						d += f'<a class="nostyling" href="{event.get_html_url}"><div class="event {event.get_html_operation_color} past-event {event.visibility}-event">{event.get_html_title}</div></a>'
					if datetime.now(timezone.utc) <= event.start_time:
						d += f'<a class="nostyling" href="{event.get_html_url}"><div class="event {event.get_html_operation_color} {event.visibility}-event">{event.get_html_title}</div></a>'

			if date.today() == date(self.year, self.month, day):
				return f"<td class='today'><div class='date'>{day}</div> {d}</td>"
			return f"<td><div class='date'>{day}</div> {d}</td>"
		return '<td></td>'

	# formats a week as a tr 
	def formatweek(self, theweek, events):
		week = ''
		for d, weekday in theweek:
			week += self.formatday(d, events)
		return f'<tr> {week} </tr>'

	# formats a month as a table
	# filter events by year and month
	def formatmonth(self, withyear=True):
		events = Event.objects.filter(start_time__year=self.year, start_time__month=self.month)

		cal = f'<table border="0" cellpadding="0" cellspacing="0" class="calendar">\n'
		cal += f'{self.formatmonthname(self.year, self.month, withyear=withyear)}\n'
		cal += f'{self.formatweekheader()}\n'
		for week in self.monthdays2calendar(self.year, self.month):
			cal += f'{self.formatweek(week, events)}\n'
		return cal


from django.dispatch import receiver

from django.db.models.signals import post_save, pre_delete, pre_save
from allianceauth.groupmanagement.models import GroupRequest
from allianceauth.authentication.models import UserProfile, CharacterOwnership, EveCharacter
from allianceauth.eveonline.evelinks.eveimageserver import  type_icon_url, character_portrait_url
from .models import Event, EventSignal
import requests
import json
import datetime
from django.utils import timezone

from .app_settings import get_site_url
from .helpers import time_helpers

from allianceauth.services.hooks import get_extension_logger
logger = get_extension_logger(__name__)

from django.utils.translation import ugettext_lazy as _

RED = 16711710
BLUE = 42751
GREEN = 6684416


@receiver(post_save, sender=Event)
def fleet_saved(sender, instance, created, **kwargs):
    if not "import" in instance.visibility:
        try:
            logger.debug("New signal fleet created for %s" % instance.title)
            url = get_site_url() + "/opcalendar/"
            main_char = instance.eve_character
            formup_system = instance.formup_system
            title = instance.title
            doctrine = instance.doctrine
            eve_time = instance.start_time
            fc = instance.fc
            col = GREEN
            message = "New Fleet Timer Posted"
            if not created:
                message = "Fleet Timer Updated"
                col = BLUE

            embed = {'title': message, 
                    'description': ("**{}** from **{}**".format(title,formup_system)),
                    'url': url,
                    'color': col,
                    "fields": [
                        {
                        "name": "FC",
                        "value": fc,
                        "inline": True
                        },
                        {
                        "name": "Doctrine",
                        "value": doctrine,
                        "inline": True
                        },
                        {
                        "name": "Eve Time",
                        "value": eve_time.strftime("%Y-%m-%d %H:%M:%S")
                        },
                        {
                        "name": "Time Until",
                        "value": time_helpers.get_time_until(eve_time)
                        }

                    ],
                    "footer": {
                        "icon_url": main_char.portrait_url_64,
                        "text": "{}  [{}]".format(main_char.character_name, main_char.corporation_ticker)
                    }
                }

            hooks = EventSignal.objects.all().select_related('webhook')
            logger.debug("Hooks OK ")
            old = datetime.datetime.now(timezone.utc) > eve_time
            logger.debug("Datetime OK")
            for hook in hooks:
                if hook.webhook.enabled:
                    if old and hook.ignore_past_fleets:
                        continue
            hook.webhook.send_embed(embed)

        except Exception as e:
            print(logger.error(e))
            pass  # shits fucked... Don't worry about it...

@receiver(pre_delete, sender=Event)
def fleet_deleted(sender, instance, **kwargs):
    if not "import" in instance.visibility:
        try:
            logger.debug("New signal fleet deleted for %s" % instance.title)
            url = get_site_url() + "/optimer/"
            main_char = instance.eve_character
            formup_system = instance.formup_system
            title = instance.title
            doctrine = instance.doctrine

            eve_time = instance.start_time

            fc = instance.fc
            message = "Fleet Removed"

            embed = {'title': message, 
                    'description': ("**{}** from **{}** has been cancelled".format(title,formup_system)),
                    'url': url,
                    'color': RED,
                    "fields": [
                        {
                        "name": "FC",
                        "value": fc,
                        "inline": True
                        },
                        {
                        "name": "Eve Time",
                        "value": eve_time.strftime("%Y-%m-%d %H:%M:%S")
                        }

                    ],
                    "footer": {
                        "icon_url": main_char.portrait_url_64,
                        "text": "{}  [{}]".format(main_char.character_name, main_char.corporation_ticker)
                    }
                }

            hooks = EventSignal.objects.all().select_related('webhook')
            old = datetime.datetime.now(timezone.utc) > eve_time

            for hook in hooks:
                if hook.webhook.enabled:
                    if old and hook.ignore_past_fleets:
                        continue
                    hook.webhook.send_embed(embed)

        except Exception as e:
            logger.error(e)
            pass  # shits fucked... Don't worry about it...


# cal/views.py
import logging

from django.contrib import messages
from datetime import datetime, date
from django.views.generic import ListView, UpdateView, DeleteView
from django.shortcuts import render, redirect
from django.http import HttpResponse, HttpResponseRedirect
from django.views import generic
from django.utils.safestring import mark_safe
from datetime import timedelta
import calendar
from django.shortcuts import get_object_or_404
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.urls import reverse_lazy

from .models import *
from .utils import Calendar
from .forms import EventForm, AddMemberForm

from django.utils.translation import ugettext_lazy as _

logger = logging.getLogger(__name__)

@login_required(login_url='signup')
def index(request):
    return HttpResponse('hello')

def get_date(req_day):
    if req_day:
        year, month = (int(x) for x in req_day.split('-'))
        return date(year, month, day=1)
    return datetime.today()

def prev_month(d):
    first = d.replace(day=1)
    prev_month = first - timedelta(days=1)
    month = 'month=' + str(prev_month.year) + '-' + str(prev_month.month)
    return month

def next_month(d):
    days_in_month = calendar.monthrange(d.year, d.month)[1]
    last = d.replace(day=days_in_month)
    next_month = last + timedelta(days=1)
    month = 'month=' + str(next_month.year) + '-' + str(next_month.month)
    return month

class CalendarView(LoginRequiredMixin, PermissionRequiredMixin, ListView):
    permission_required = 'opcalendar.basic_access'
    login_url = 'signup'
    model = Event
    template_name = 'opcalendar/calendar.html'

    def get_context_data(self, **kwargs):
        user = self.request.user
        #user_permissions = self.request.user.has_perm('app.get_main_view')
        context = super().get_context_data(**kwargs)
        d = get_date(self.request.GET.get('month', None))
        cal = Calendar(d.year, d.month, user)
        html_cal = cal.formatmonth(withyear=True)
        context['category'] = EventCategory.objects.all()
        context['calendar'] = mark_safe(html_cal)
        context['prev_month'] = prev_month(d)
        context['next_month'] = next_month(d)
        context['legend'] = "test"
        return context

@login_required(login_url='signup')
@permission_required("opcalendar.create_event")
def create_event(request):    
    form = EventForm(request.POST or None)
    if request.POST and form.is_valid():
        # Get character
        character = request.user.profile.main_character
        operation_type = form.cleaned_data['operation_type']
        title = form.cleaned_data['title']
        doctrine = form.cleaned_data['doctrine']
        formup_system = form.cleaned_data['formup_system']
        description = form.cleaned_data['description']
        start_time = form.cleaned_data['start_time']
        end_time = form.cleaned_data['end_time']
        fc = form.cleaned_data['fc']
        visibility = form.cleaned_data['visibility']
        Event.objects.get_or_create(
            user=request.user,
            operation_type=operation_type,
            title=title,
            doctrine=doctrine,
            formup_system=formup_system,
            description=description,
            start_time=start_time,
            end_time=end_time,
            eve_character = character,
            fc=fc,
            visibility=visibility
        )
        messages.success(request, _('Event %(opname)s created for %(date)s.') % {"opname": title,"date": start_time.strftime("%Y-%m-%d %H:%M")})
        return HttpResponseRedirect(reverse('opcalendar:calendar'))
    return render(request, 'opcalendar/event-add.html', {'form': form})



@login_required(login_url='signup')
@permission_required("opcalendar.basic_access")
def event_details(request, event_id):
    event = Event.objects.get(id=event_id)
    context = {
        'event': event
    }
    if request.user.has_perm('opcalendar.view_public') and event.visibility == "public" or event.visibility == "import":
        return render(request, 'opcalendar/event-details.html', context)
    elif request.user.has_perm('opcalendar.view_member') and event.visibility == "member":
        return render(request, 'opcalendar/event-details.html', context) 
    else:
        return redirect('opcalendar:calendar')

class EventEdit(generic.UpdateView):
    permission_required = "opcalendar.create_event"
    model = Event
    fields = ['operation_type', 'title', 'doctrine', 'formup_system', 'description', 'start_time','end_time','fc', 'visibility']
    template_name = 'opcalendar/event-edit.html'

@login_required(login_url='signup')
@permission_required("opcalendar.manage_event")
def add_eventmember(request, event_id):
    forms = AddMemberForm()
    if request.method == 'POST':
        forms = AddMemberForm(request.POST)
        if forms.is_valid():
            member = EventMember.objects.filter(event=event_id)
            event = Event.objects.get(id=event_id)
            if member.count() <= 9:
                user = forms.cleaned_data['user']
                EventMember.objects.create(
                    event=event,
                    user=user
                )
                return redirect('opcalendar:calendar')
            else:
                print('--------------User limit exceed!-----------------')
    context = {
        'form': forms
    }
    return render(request, 'opcalendar/add_member.html', context)



@login_required
@permission_required('opcalendar.create_event')
def EventDeleteView(request, event_id):
    logger.debug("remove_optimer called by user %s for operation id %s" % (request.user, event_id))
    op = get_object_or_404(Event, id=event_id)
    op.delete()
    logger.info("Deleting optimer id %s by user %s" % (event_id, request.user))
    messages.error(request, _('Removed event %(opname)s.') % {"opname": op.title})
    return redirect("opcalendar:calendar")